package net.lecnam.info.ex3;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.logging.Logger;
import net.lecnam.info.ex3.Answer.*;
import net.lecnam.info.ex3.Request.*;

public class Client implements Runnable {

	ObjectInputStream in;
	ObjectOutputStream out;

	Client(ObjectInputStream in, ObjectOutputStream out) {
		this.in = in;
		this.out = out;
	}

	static final Logger logger = Logger.getLogger("server");

	public int size() {
		try {
			Request request = new GetSize();
			out.writeObject(request);
			Answer answer = (Answer)in.readObject();
			logger.info("Answer: " + answer);
			switch (answer) {
				case Size(var s) -> {
					return s;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (Exception ex) {
			throw new Error(ex);
		}
	}

	public String get(int index) {
		try {
			Request request = new GetValue(index);
			out.writeObject(request);
			Answer answer = (Answer)in.readObject();
			logger.info("Answer: " + answer);
			switch (answer) {
				case Value(var elt) -> {
					return elt;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (Exception ex) {
			throw new Error(ex);
		}
	}

	public String set(int index, String element) {
		try {
			Request request = new Set(index, element);
			out.writeObject(request);
			Answer answer = (Answer)in.readObject();
			logger.info("Answer: " + answer);
			switch (answer) {
				case OldValue(var elt) -> {
					return elt;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (Exception ex) {
			throw new Error(ex);
		}
	}

	public String remove(int index) {
		try {
			Request request = new Remove(index);
			out.writeObject(request);
			Answer answer = (Answer)in.readObject();
			logger.info("Answer: " + answer);
			switch (answer) {
				case OldValue(var elt) -> {
					return elt;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (Exception ex) {
			throw new Error(ex);
		}
	}

	public void add(int index, String element) {
		try {
			Request request = new Add(index, element);
			out.writeObject(request);
			Answer answer = (Answer)in.readObject();
			logger.info("Answer: " + answer);
			switch (answer) {
				case Done() -> {
					return;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (Exception ex) {
			throw new Error(ex);
		}
	}

	public void stop() {
		try {
			Request request = new Stop();
			out.writeObject(request);
		} catch (Exception ex) {
			throw new Error(ex);
		}
	}

	@Override
	public void run() {
		int s = size();
		System.out.println("size: " + s);
		for (int i = 0; i < s; i++) {
			String elt = get(i);
			System.out.println(elt);
		}
		set(4, "b14");
		for (int i = 0; i < s; i++) {
			String elt = get(i);
			System.out.println(elt);
		}
		add(2, "c89");
		for (int i = 0; i < s; i++) {
			String elt = get(i);
			System.out.println(elt);
		}
		stop();
	}

	public static void main(String[] args) throws UnknownHostException, IOException {
		System.out.println("Client is running");
		Socket s = new Socket("localhost", 55000);
		System.out.println("Connection established");
		System.out.println("Local port:" + s.getLocalPort());
		System.out.println("Local IP:" + s.getLocalAddress());
		System.out.println("Remote port:" + s.getPort());
		System.out.println("Remote IP:" + s.getInetAddress());
		//
		InputStream is = s.getInputStream();
		OutputStream os = s.getOutputStream();

		ObjectOutputStream out = new ObjectOutputStream(os);
		ObjectInputStream in = new ObjectInputStream(is);

		Client client = new Client(in, out);
		client.run();

		System.out.println("Closing");
		s.close();

	}

}
