package net.lecnam.info.ex3;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import net.lecnam.info.ex3.Answer.*;
import net.lecnam.info.ex3.Request.*;

public class Server implements Runnable {

	ObjectInputStream in;
	ObjectOutputStream out;
	List<String> l;

	static final Logger logger = Logger.getLogger("server");

	Server(ObjectInputStream in, ObjectOutputStream out, List<String> l) {
		this.in = in;
		this.out = out;
		this.l = new ArrayList<>(l);
	}

	@Override
	public void run() {
		try {
			boolean done = false;
			while (!done) {
				Request request = (Request)in.readObject();
				logger.info("Request: " + request);
				switch (request) {
					case Stop() -> {
						done = true;
					}
					case GetSize() -> {
						int s = l.size();
						Answer answer = new Size(s);
						out.writeObject(answer);
					}
					case GetValue(var index) -> {
						String elt = l.get(index);
						Answer answer = new Value(elt);
						out.writeObject(answer);
					}
					case Set(var index, var element) -> {
						String old = l.set(index, element);
						Answer answer = new OldValue(old);
						out.writeObject(answer);
					}
					case Add(var index, var element) -> {
						l.add(index, element);
						Answer answer = new Done();
						out.writeObject(answer);
					}
					case Remove(var index) -> {
						String old = l.remove(index);
						Answer answer = new OldValue(old);
						out.writeObject(answer);
					}
				}
			}
		} catch (Exception ex) {
			throw new Error(ex);
		}
	}

	public static void main(String[] args) throws IOException {
		System.out.println("Server is running");
		ServerSocket ss = new ServerSocket(55000);
		System.out.println("Waiting for a client");
		Socket s = ss.accept();
		System.out.println("Client connected");
		System.out.println("Local port:" + s.getLocalPort());
		System.out.println("Local IP:" + s.getLocalAddress());
		System.out.println("Remote port:" + s.getPort());
		System.out.println("Remote IP:" + s.getInetAddress());
		//
		InputStream is = s.getInputStream();
		OutputStream os = s.getOutputStream();

		ObjectOutputStream out = new ObjectOutputStream(os);
		ObjectInputStream in = new ObjectInputStream(is);

		List<String> l = List.of("a1", "a3", "a5", "a7", "a9", "a11", "a13", "a17");

		Server server = new Server(in, out, l);
		server.run();

		System.out.println("Closing");
		s.close();
		ss.close();
	}

}
